# -*- coding: utf-8 -*-

import json
import time
import pymysql
import logging
import logging.config
from artix.filequeue.filequeuemanager import FileQueueManager
from artix.database.info import DBInfo

logging.config.fileConfig('maint_log.ini')
log = logging.getLogger('su.artix.pricedeleteservice')
cashRegPath = '/linuxcash/cash/data/cash.reg'
configPath = '/linuxcash/cash/conf/maint.json'


def chunks(chunkable, n):
    for i in range(0, len(chunkable), n):
        yield chunkable[i:i+n]


def createTransaction(log, commandList, cashCode):
    queueName = 'artix.loaddict.cash'
    try:
        fileQueueManager = FileQueueManager(queueRoot='/linuxcash/cash/data/queues', log=log)
        fileQueueManager.removeUncompletedTransactions(queueName, 'maint')
        transactionId = fileQueueManager.beginTransaction(queueName)
        fileQueueManager.enqueue(queueName, transactionId, {'properties': {'type': 'datagram'}, 'content': commandList})
        endTransProperties = {
            'type': 'control',
            'control': 'end_tran',
            'cashCode': cashCode,
            'statusServer': 'localhost:8080',
            'source': 'maint',
            'transactionId': transactionId,
            'objectsCount': len(commandList)
            }
        fileQueueManager.endTransaction(queueName, transactionId, {'properties': endTransProperties})
    except Exception as e:
        log.error('Ошибка: %s' % str(e))


def readJson(path):
    try:
        with open(path) as data_file:
            data = json.load(data_file)
        return data
    except Exception as e:
        raise Exception("Failed to read %s. %s" % (path, str(e)))


def calculateData():
    conn = None
    cur = None
    try:
        log.info('Удаляем старые переоценки')
        cashCode = str(readJson(cashRegPath)['cashCode'])
        data = readJson(configPath)
        resultCount = data['prices']['pricesCount'] if 'prices' in data and 'pricesCount' in data['prices'] else 500
        timeout = data['prices']['timeout'] if 'prices' in data and 'timeout' in data['prices'] else 240
        log.info('Начало выборки данных из бд')
        dbInfo = DBInfo()
        conn = pymysql.connect(host='localhost', user=dbInfo.user, password=dbInfo.password, database='dictionaries', autocommit=True)
        cur = conn.cursor()
        cur.execute('set names utf8')
        cur.execute("SELECT code FROM tmc")
        codeList = [item[0] for item in cur.fetchall()]
        log.info('Выбрали все коды из таблицы tmc')
        cur.execute("SELECT barcode FROM barcodes")
        codeList.extend([item[0] for item in cur.fetchall()])
        log.info('Выбрали все штрих-коды из таблицы barcodes')

        cur.execute("SELECT DISTINCT(barcode) FROM dictionaries.prices WHERE effectivedateend < now()")
        result = [{'command': 'deleteOverduePrices', 'barcode': str(item[0])} for item in cur.fetchall()]
        log.info(str(result))
        for data in chunks(result, resultCount):
            log.info('Посылаем %d команд в nes' % len(data))
            createTransaction(log, data, cashCode)
            time.sleep(timeout)

        commandList = []
        for row in chunks(codeList, 50):
            row = filter(None, row)
            query_values = ', '.join("'%s'" % it for it in row)
            query = "SELECT barcode, max(effectivedate), effectivedateend FROM dictionaries.prices \
                    WHERE barcode IN (%s) AND effectivedate < now() AND (effectivedateend > now() OR effectivedateend IS NULL) \
                    GROUP BY barcode, effectivedateend HAVING count(barcode) > 1" % str(query_values)
            cur.execute(query)
            result = [
                {'command': 'deletePriceByDate',
                 'barcode': str(item[0]),
                 'effectivedate': str(item[1]),
                 'effectivedateend': str(item[2]) if item[2] else None}
                for item in cur.fetchall()]

            if result:
                log.info(str(result))
                commandList.extend(result)
                log.info('Сформировали %d команд для nes. Общий размер пакета %d' % (len(result), len(commandList)))

                if len(commandList) >= resultCount:
                    for data in chunks(commandList, resultCount):
                        log.info('Посылаем %d команд в nes' % len(data))
                        createTransaction(log, data, cashCode)
                        time.sleep(timeout)
                    commandList = []
        if commandList:
            createTransaction(log, commandList, cashCode)
    except Exception as e:
        log.error('Ошибка : %s' % e)
    finally:
        if conn and conn.open:
            if cur:
                cur.close()
            conn.close()


def getTPricesDeleteCommand(cur, tindex, bcode):
    """
    Получить команду для на удаление для пары bcode и tindex
    :param cur: курсор
    :param tindex: индекс
    :param bcode: штрих-код
    :return: команда на удаление
    """
    # Выбираем ближаюшую к текущей прошедшую дату effectivedate для пары bcode и tindex
    cur.execute('SELECT effectivedate FROM dictionaries.tprices '
                'WHERE effectivedate < now() AND bcode = "%s" AND tindex = %d '
                'ORDER BY effectivedate DESC LIMIT 1' % (bcode, tindex))
    item = cur.fetchone()
    # После удаления должна остаться одна запись, так что удаляем все записи с датой меньше выбранной
    return {'command': 'deleteOutdatedAdditionalPrice',
            'pricecode': tindex,
            'barcode': bcode,
            'effectivedate': str(item[0])}


def removeOverdueAdditionalPrices():
    """
    Удалить старые доп. цены из таблицы tprices, с сохранением одной записи для каждой пары bcode и tindex
    """
    conn = None
    cur = None
    try:
        log.info('Удаляем старые доп. цены')
        cashCode = str(readJson(cashRegPath)['cashCode'])
        conf = readJson(configPath)
        resultCount = conf['prices']['pricesCount'] if 'prices' in conf and 'pricesCount' in conf['prices'] else 500
        timeout = conf['prices']['timeout'] if 'prices' in conf and 'timeout' in conf['prices'] else 240
        dbInfo = DBInfo()
        conn = pymysql.connect(host=dbInfo.host, port=dbInfo.port, user=dbInfo.user, password=dbInfo.password, database='dictionaries', autocommit=True)
        cur = conn.cursor()
        cur.execute('set names utf8')
        log.info("Выбраем пары tindex и bcode для удаления устаревших доп. цен")
        # Ищем уникальные пары bcode и tindex с количеством записей больше 1
        cur.execute('SELECT tindex, bcode FROM dictionaries.tprices '
                    'WHERE effectivedate < now() GROUP BY tindex, bcode HAVING COUNT(*) > 1')
        log.info("Формируем список команд на удаления доп. цен")
        commands = [getTPricesDeleteCommand(cur, int(item[0]), str(item[1])) for item in cur.fetchall()]
        log.info("Список команд сформирован")
        if not commands:
            log.info("Нет записей для удаления")
            return
        for cmds in chunks(commands, resultCount):
            log.info('Посылаем %d команд в nes' % len(cmds))
            createTransaction(log, cmds, cashCode)
            time.sleep(timeout)
    except Exception as e:
        log.error('Ошибка : %s' % e)
    finally:
        if conn and conn.open:
            if cur:
                cur.close()
            conn.close()


def main():
    log.info('Сервис maint_prices начал работу')
    calculateData()
    removeOverdueAdditionalPrices()
    log.info('Сервис maint_prices закончил работу')


if __name__ == '__main__':
    main()
